#!/usr/bin/perl
# $Id: 93_update_disks_info.pl
#
# makes sure a Disk object exists for each internal logical partition, 
# and that each Disk's total space and used space are up to date
#

use strict;
use lib qw(/usr/sausalito/perl /usr/sausalito/handlers/base/disk);
use CCE;
use Base::Disk qw(disk_getmounts disk_get_usage_info);
use Base::HomeDir;

my $cce = new CCE('Domain' => 'base-disk');

$cce->connectuds();

my $mounts = disk_getmounts();

# Find all existing DISK objects in CODB and set 'mounted' to '0'. That way we hide
# all disks and partitions which at one point were present, but are no longer here.
my @oldDiskOIDs = $cce->findx('Disk');
for my $entry (@oldDiskOIDs) {
    (my $ok) = $cce->set($entry, '', { 'mounted' => "0" });
}

for my $partition (keys %$mounts) {

    # this script only deals with internal disks
    next unless ($partition =~ /^\/dev/);

    next if ($partition =~ /^\/dev\/ploop(.*)$/);

    my ($oid) = $cce->find("Disk", { 'device' => $partition });

    # get disk usage information
    my $info = disk_get_usage_info($partition);

    # Get fsType:
    my $fsType = 'ext3';
    my $mp = $info->{$partition}->{MountPoint};
    if ((-f "/etc/fstab") && ($mp ne "/")) {
        $fsType = `cat /etc/fstab |grep $mp|awk '{print \$3}'`;
        chomp($fsType);
    }

    if ($oid) {
        my $isHomePartition = '0';
        if ($info->{$partition}->{MountPoint} =~ /$Base::HomeDir::HOME_ROOT/) {
            $isHomePartition = '1';
        }

        $cce->set($oid, "", {   
                            'device' => $partition,
                            'mountPoint' => $info->{$partition}->{MountPoint},
                            'total' => $info->{$partition}->{Total},
                            'used' => $info->{$partition}->{Used},
                            'mounted' => 1,
                            'fsType' => $fsType,
                            'isHomePartition' => $isHomePartition
                            });
    }
    else {
        my $partition_info = { 
                            'device' => $partition,
                            'mountPoint' => $info->{$partition}->{MountPoint},
                            'total' => $info->{$partition}->{Total},
                            'used' => $info->{$partition}->{Used},
                            'mounted' => 1,
                            'internal' => 1,
                            'fsType' => $fsType,
                        };

        if ($info->{$partition}->{MountPoint} =~ /$Base::HomeDir::HOME_ROOT/) {
            $partition_info->{isHomePartition} = 1;
        }
       
        my ($ok) = $cce->create("Disk", $partition_info);

        if (not $ok) {
            $cce->bye('FAIL', 'cantCreateDisk', { 'device' => $partition });
            exit(1);
        }
    }
}

# Find /home in CCE and if it is not there, create a simulated /home instead.
# We really need one (at least simulated) or we cannot create Vsites:

my @oids = $cce->find('Disk', { 'mountPoint' => '/home' });
if ($#oids < 0) {  

            # Populate CCE:   
            my ($ok) = $cce->create('Disk', {
                'isHomePartition' => '1',
                'mounted' => '1',
                'fsType' => 'ext3',
                'mountPoint' => '/home', 
                'CLASS' => 'Disk',
                'quota' => '0',
                'uuid' => '',
                'device' => '/dev/simfs',
                'label' => '',
                'check' => '0',
                'internal' => '1',
                'mount' => '0'
                 });
}

# Destroy Disk object '/var/freespace' if it exists:
my @oldvar = $cce->findx('Disk', { 'mountPoint' => '/var/freespace' });
for my $entry (@oldvar) {

    # Unmount and lvmremove /var/freespace and comment it out in /etc/fstab:
    system("/bin/umount /var/freespace");
    system("/sbin/lvremove /dev/sysvg/freespace");
    system("/usr/bin/perl -pi -e 's/^\/dev\/mapper\/sysvg-free/#\/dev\/mapper\/sysvg-free/' /etc/fstab > /dev/null 2>&1");

    # Destroy disk object in CODB:
    my ($ok) = $cce->destroy($entry);
}

$cce->bye('SUCCESS');
exit(0);

# 
# Copyright (c) 2016 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2016 Team BlueOnyx, BLUEONYX.IT
# Copyright (c) 2003 Sun Microsystems, Inc. 
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 